 
%addpath(sprintf('%s/ProgTlbxMatlab/',pwd));
addpath(sprintf('%s/Stationary/SourceGen_Filt/',pwd));
addpath(sprintf('%s/Stationary/',pwd));

clear; close all;

disp('-----------------------------------------------');
disp('Convolutive source separation Toolbox demo');
disp('-----------------------------------------------');
fprintf('In this demo, the signals are real and stationnary; the mixture is convolutive.\n');
fprintf('You may test separation methods respectively designed for:\n\n');
fprintf('1) convolutive mixture, stationary sources, real valued case\n\t Maximized contrast: kurtosis based\n\t Method: gradient, optimal step\n');
fprintf('2) convolutive mixture, stationary sources, complex valued case\n\t (also valid in the real valued case)\n\t Maximized contrast: kurtosis based\n\t Method: gradient, optimal step\n');
fprintf('3) convolutive mixture, stationary sources, complex valued case\n\t (also valid in the real valued case)\n\t Maximized contrast: quadratic contrast\n\t Method: SVD based\n');
fprintf('4) convolutive mixture, stationary sources, complex valued case\n\t (also valid in the real valued case)\n\t Maximized contrast: quadratic contrast\n\t Method: gradient, optimal step\n');
fprintf('5) convolutive mixture, stationary sources, complex valued case\n\t (also valid in the real valued case)\n\t Maximized contrast: cubic contrast\n\t Method: HOPM based\n');
fprintf('6) convolutive mixture, stationary sources, complex valued case\n\t (also valid in the real valued case)\n\t Maximized contrast: quadratic contrast with reference (semi-blind)\n\t Method: SVD based\n\t Reference = sign(s) with prob 0.9\n');
fprintf('7) convolutive mixture, cyclostationary sources, complex valued case\n\t (also valid in the real valued case and in the stationnary case)\n\t Maximized contrast: kurtosis based\n');
choix_utilisateur = input('Your choice? ');

disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp('Randomly generating a mixture and separating the corresponding sources...');
disp('Please wait until display of the separation results...');

%---generate a mixture-----------------------------------------------------
Nsample = 5000;     %number of samples (of the sources)
Nsource = 3;        %number of sources
Nobserv = Nsource+1;%number of sensors
L = 3;              %length of the mixing filter
D=Nsource*(L-1);    %length of the separating filter (set to the minimum 
                    %value so as to ensure invertibility of the mixture)
                    
s = generSources(Nsource,Nsample,1);%random generation of the sources
M = randn(Nobserv,Nsource,L);       %random generation of the mixing filter
x = MIMOfilt(M,s);                  %convolutive mixing of the sources

%--- set a Param structure for use with Deflation.m------------------------
%--- (parameters are set depending on the chosen method)-------------------
switch(choix_utilisateur),
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    case 1,
MyParameters.LFilter            = D;      %length of the mixing filter
MyParameters.Seuil              = 0;      %threshold value for convergence 
                                          %of the gradient. If the norm
                                          %of the parameters vector change
                                          %is less than this value, no other 
                                          %gradient update is peformed.
MyParameters.NbIterationMax     = 1000;   %maximum gradient iterations
MyParameters.Nsources           = Nsource;%number of sources to be extracted
MyParameters.SeuilStopAlgo      = 0;      %threshold value on the power of 
                                          %the mixture to determine whether
                                          %there are other sources or not. 
                                          %Under this value, it is 
                                          %considered that there is no more
                                          %source to be extracted and no
                                          %other deflation is performed. 

MyParameters.LSubtractFiltreAC  = L-1;    %length of the anti-causal part
                                          %of the subtracting filter in 
                                          %the deflation step
MyParameters.LSubtractFiltreC   = D+L-2;  %length of the causal part of  
                                          %the subtracting filter in the 
                                          %deflation step

MyParameters.Method = 'kurtosisRealValued';%chosen separation method
MyParameters.GradientStep = 'optim';      %gradient update by an optimal 
                                          %step size
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    case 2,
MyParameters.LFilter            = D;      %length of the mixing filter
MyParameters.Seuil              = 0;      %threshold value for convergence 
                                          %of the gradient. If the norm
                                          %of the parameters vector change
                                          %is less than this value, no other 
                                          %gradient update is peformed.
MyParameters.NbIterationMax     = 1000;   %maximum gradient iterations
MyParameters.Nsources           = Nsource;%number of sources to be extracted
MyParameters.SeuilStopAlgo      = 0;      %threshold value on the power of 
                                          %the mixture to determine whether
                                          %there are other sources or not. 
                                          %Under this value, it is 
                                          %considered that there is no more
                                          %source to be extracted and no
                                          %other deflation is performed. 

MyParameters.LSubtractFiltreAC  = L-1;    %length of the anti-causal part
                                          %of the subtracting filter in 
                                          %the deflation step
MyParameters.LSubtractFiltreC   = D+L-2;  %length of the causal part of  
                                          %the subtracting filter in the 
                                          %deflation step

MyParameters.Method = 'kurtosis';%chosen separation method
MyParameters.GradientStep = 'optim';      %gradient update by an optimal 
                                          %step size
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    case 3,
MyParameters.LFilter            = D;      %length of the mixing filter
MyParameters.Seuil              = 0;      %threshold value for convergence 
                                          %of the gradient. If the norm
                                          %of the parameters vector change
                                          %is less than this value, no other 
                                          %gradient update is peformed.
MyParameters.NbIterationMax     = 1000;   %maximum gradient iterations
MyParameters.Nsources           = Nsource;%number of sources to be extracted
MyParameters.SeuilStopAlgo      = 0;      %threshold value on the power of 
                                          %the mixture to determine whether
                                          %there are other sources or not. 
                                          %Under this value, it is 
                                          %considered that there is no more
                                          %source to be extracted and no
                                          %other deflation is performed. 

MyParameters.LSubtractFiltreAC  = L-1;    %length of the anti-causal part
                                          %of the subtracting filter in 
                                          %the deflation step
MyParameters.LSubtractFiltreC   = D+L-2;  %length of the causal part of  
                                          %the subtracting filter in the 
                                          %deflation step
%---additionnal parameters required for quadratic contrasts----------------
MyParameters.NsourcesEffectiv = Nsource;%number of sources effectively 
                                        %present in the mixture (assumed to 
                                        %be known)
MyParameters.Quadratic.L = L;    %length of the mixing filter (assumed to 
                                 %be known)
MyParameters.Quadratic.Nifix = 5;%number of "fixed point" iterations (see 
                                 %the method descriptions for more details)

MyParameters.Method = 'quadSVD';          %chosen separation method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    case 4,
MyParameters.LFilter            = D;      %length of the mixing filter
MyParameters.Seuil              = 0;      %threshold value for convergence 
                                          %of the gradient. If the norm
                                          %of the parameters vector change
                                          %is less than this value, no other 
                                          %gradient update is peformed.
MyParameters.NbIterationMax     = 1000;   %maximum gradient iterations
MyParameters.Nsources           = Nsource;%number of sources to be extracted
MyParameters.SeuilStopAlgo      = 0;      %threshold value on the power of 
                                          %the mixture to determine whether
                                          %there are other sources or not. 
                                          %Under this value, it is 
                                          %considered that there is no more
                                          %source to be extracted and no
                                          %other deflation is performed. 

MyParameters.LSubtractFiltreAC  = L-1;    %length of the anti-causal part
                                          %of the subtracting filter in 
                                          %the deflation step
MyParameters.LSubtractFiltreC   = D+L-2;  %length of the causal part of  
                                          %the subtracting filter in the 
                                          %deflation step
%---additionnal parameters required for quadratic contrasts----------------
MyParameters.NsourcesEffectiv = Nsource;%number of sources effectively 
                                        %present in the mixture (assumed to 
                                        %be known)
MyParameters.Quadratic.L = L;    %length of the mixing filter (assumed to 
                                 %be known)
MyParameters.Quadratic.Nifix = 5;%number of "fixed point" iterations (see 
                                 %the method descriptions for more details)

MyParameters.Method = 'quadGrad';          %chosen separation method
MyParameters.GradientStep = 'optim';      %gradient update by an optimal 
                                          %step size
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    case 5,
MyParameters.LFilter            = D;      %length of the mixing filter
MyParameters.Seuil              = 1e-9;      %threshold value for convergence 
                                          %of the HOPM. If the norm
                                          %of the parameters vector change
                                          %is less than this value, no other 
                                          %HOPM iteration is peformed.
MyParameters.USeuil              = 1;     %use of threshold of maximum number of iterations.
MyParameters.NbIterationMax     = 100;   %maximum gradient iterations or HOPM iterations
MyParameters.Nsources           = Nsource;%number of sources to be extracted
MyParameters.SeuilStopAlgo      = 0;      %threshold value on the power of 
                                          %the mixture to determine whether
                                          %there are other sources or not. 
                                          %Under this value, it is 
                                          %considered that there is no more
                                          %source to be extracted and no
                                          %other deflation is performed. 

MyParameters.LSubtractFiltreAC  = L-1;    %length of the anti-causal part
                                          %of the subtracting filter in 
                                          %the deflation step
MyParameters.LSubtractFiltreC   = D+L-2;  %length of the causal part of  
                                          %the subtracting filter in the 
                                          %deflation step
%---additionnal parameters required for quadratic contrasts----------------
MyParameters.NsourcesEffectiv = Nsource;%number of sources effectively 
                                        %present in the mixture (assumed to 
                                        %be known)
MyParameters.Cubic.L = L;    %length of the mixing filter (assumed to 
                                 %be known)
MyParameters.Cubic.Nifix = 3;%number of "fixed point" iterations (see 
                                 %the method descriptions for more details)

MyParameters.Method = 'cubiHOPM';          %chosen separation method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    case 6,
MyParameters.LFilter            = D;      %length of the mixing filter
MyParameters.Seuil              = 0;      %threshold value for convergence 
                                          %of the gradient. If the norm
                                          %of the parameters vector change
                                          %is less than this value, no other 
                                          %gradient update is peformed.
MyParameters.NbIterationMax     = 1000;   %maximum gradient iterations
MyParameters.Nsources           = Nsource;%number of sources to be extracted
MyParameters.SeuilStopAlgo      = 0;      %threshold value on the power of 
                                          %the mixture to determine whether
                                          %there are other sources or not. 
                                          %Under this value, it is 
                                          %considered that there is no more
                                          %source to be extracted and no
                                          %other deflation is performed. 

MyParameters.LSubtractFiltreAC  = L-1;    %length of the anti-causal part
                                          %of the subtracting filter in 
                                          %the deflation step
MyParameters.LSubtractFiltreC   = D+L-2;  %length of the causal part of  
                                          %the subtracting filter in the 
                                          %deflation step
%---additionnal parameters required for quadratic contrasts----------------
MyParameters.NsourcesEffectiv = Nsource;%number of sources effectively 
                                        %present in the mixture (assumed to 
                                        %be known)
MyParameters.Quadratic.L = L;    %length of the mixing filter (assumed to 
                                 %be known)
MyParameters.Reference = sign(s).* sign(rand(size(s))-0.1);
                                         %reference signal
MyParameters.Method = 'refSVD';          %chosen separation method        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    case 7,
MyParameters.LFilter            = D;      %length of the mixing filter
MyParameters.Seuil              = 0;      %threshold value for convergence 
                                          %of the gradient. If the norm
                                          %of the parameters vector change
                                          %is less than this value, no other 
                                          %gradient update is peformed.
MyParameters.NbIterationMax     = 1000;   %maximum gradient iterations
MyParameters.Nsources           = Nsource;%number of sources to be extracted
MyParameters.SeuilStopAlgo      = 0;      %threshold value on the power of 
                                          %the mixture to determine whether
                                          %there are other sources or not. 
                                          %Under this value, it is 
                                          %considered that there is no more
                                          %source to be extracted and no
                                          %other deflation is performed. 

MyParameters.LSubtractFiltreAC  = L-1;    %length of the anti-causal part
                                          %of the subtracting filter in 
                                          %the deflation step
MyParameters.LSubtractFiltreC   = D+L-2;  %length of the causal part of  
                                          %the subtracting filter in the 
                                          %deflation step

MyParameters.Method = 'cyclostat';%chosen separation method
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---perform separation-----------------------------------------------------
[y,Contribution,g] = Deflation(x,MyParameters,[]);

%---evaluate the result quality and display--------------------------------
decalmax = 2*(L+D-2); %corrected 19/06/2009 max(L-1,D-1);
[MSE,news,newy] = elimamb_g3(s,y,decalmax); %compute the mean square error

disp('Mean square error on the respective sources:'); disp(MSE);
subplot(3,1,1); plot(news(:,:).'); title('Original sources'); axis([101 150 -2 2]);
subplot(3,1,2); plot(x(:,:).'); title('Observations'); axis([101 150 -10 10]);
subplot(3,1,3); plot(newy(:,:).'); title('Reconstructed sources'); axis([101 150 -2 2]);



disp('-----------------------------------------------');
disp('END of convolutive source separation Toolbox demo');
disp('-----------------------------------------------');

% rmpath(sprintf('%s/Stationary/SourceGen_Filt/',pwd));
% rmpath(sprintf('%s/Stationary/',pwd));
