function [U,loopend] = hopm(A,loop,epsilon,stop)
% function  [U,loopend] = hopm(A,loop,epsilon,stop)
%
%           HO power method for the computation of the best rank-1 approximation of a 3rd order tensor
%
% A      : 3rd order tensor
% loop   : set number max of loop
% epsilon: epsilon if stopping criterion
% stop   : =0 no stopping criterion (max loop)
% 
% U      : Vector of the rank-1 approximation
% loopend: number of loop for convergence


S=size(A);
O=ndims(A);

Odim=1:O;

[dim1,i1]=max(S);
[dim2,i2]=min(S);
dim2=prod(S)/S(i2);

% termination criterion number of loop, or epsilon
U=zeros(max(S),O,loop);
An=zeros(dim1,dim2,O);
lambda=zeros(loop,O);
lambdaf=zeros(loop,1);

% Initialization with dominant left singular vector of A(k)
[V] = hosvd(A);
for i=1:O
    U(1:S(i),i,1)=V(1:S(i),i);
    ndim(i)=prod(S)/S(i);
    An(1:S(i),1:ndim(i),i)=unfolding(A,i);
end


At=A-outer_product(U(:,:,1),S);
At=At.*conj(At);
for i=1:O
    tmp=sum(At);
    At=tmp;
end
lambdaf(1)=sqrt(tmp);

loopend=loop;

% Main Loop
for k=1:(loop-1)
    er=0;
    for i=1:O
        nOdim=circshift(Odim,[1 -(i-1)]);
        
        V1=conj(squeeze(U(1:S(nOdim(2)),nOdim(2),k+(nOdim(2)<i))));        
        V2=V1;
        for j=2:O-1
            Qe=conj(squeeze(U(1:S(nOdim(j+1)),nOdim(j+1),k+(nOdim(j+1)<i))));
            V2=kron(V1,Qe);
            V1=V2;
        end
        
        U(1:S(i),i,k+1)=squeeze(An(1:S(i),1:ndim(i),i))*V2;
        lambda(k+1,i)=norm(squeeze(U(1:S(i),i,k+1)));
        U(1:S(i),i,k+1)=U(1:S(i),i,k+1)/lambda(k+1,i);
        
        er=er+norm(squeeze(U(1:S(i),i,k+1))-squeeze(U(1:S(i),i,k)),'fro')/norm(squeeze(U(1:S(i),i,k+1)),'fro');
    end
        At=A-outer_product(U(:,:,k+1),S);
        At=At.*conj(At);
        for i=1:O
            tmp=sum(At);
            At=tmp;
        end
        lambdaf(k+1)=sqrt(tmp);
        
        loopend=loop;
        
        if stop==0

        else
            if (er<epsilon)
                loopend=k+1;
                break;
            end
        end
        
end

lambdaif=lambdaf(1:loopend);
U=squeeze(U(:,:,loopend));
 

