function [W,crit] = GradientKurtR(epsilon,Nmax,y,f_ini,choix_mu,varargin);

% [W,crit] = GradientKurtR(epsilon,Nmax,y,f_ini,choix_mu,varargin);
%
% Gradient optimization of a kurtosis based contrast function (real valued
% signals)
% GradientKurtR performs a gradient ascent optimization of the contrast
% given by the function CostKurtR
%
% Inputs:
% -------
% epsilon = gradient stopping criterion (minimum update difference)
% Nmax = maximum number of gradient iterations
% y = (Nx x Nsample) matrix containing the Nx observations of the mixture
% f_ini = (1 x D*Nx) vector: initial value for the gradient search of the
%   separating filter
% choix_mu : determines the update strategy
%   choix_mu = 'optim' : optimal step size (not yet implemented)
%   choix_mu = 'adapt' : adaptive step size. The step size is multiplied by
%       1/2 each time the criterion does not increase.
%   choix_mu = numerical value: fixed step size set to the given value
% varargin : if choix_mu = 'adapt', varargin = initial step size value (set
%   to 1 by default)
%
% Outputs: 
% -------
% W = (1 x D*Nx) vector: separating filter
% crit = final value of the contrast
%
% -------
% Author: Marc Castella
% Created: ??/2001
% Last modified: 04/07/2006
% Contact: marc.castella@int-evry.fr

% Autres rq:
% 06/02/02 change l'ordre dans la matrice Y
% varargin = valeur initiale de mu si choix_mu = 'adapt' MC 18/11/2002
% f = vecteur ligne (transposition conjuguee avant fin pgme) 09/05/2003
%   %doc a mettre a jour!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% 29/10/03 nombre mini d'iteration gradient=50
% 30/10/03 changement du critre arret: depend norm(f-newf)/length(f) et plus norm(dc_df)/length(dc_df)

% Initialisation de diverses matrices

Nobserv = size(y,1);

Y = [];
D = length(f_ini)/Nobserv;
for ii = 1:D,
%    Y = [ Y; y(:,ii:end+ii-D)];
    Y = [ y(:,ii:end+ii-D); Y];
end;
R = 1/length(Y)*Y*Y'; %%%R = cov(Y');
f = f_ini'; %f = f/sqrt(f'*R*f);

% Algo du gradient
iteration = 0;
dc_df = 1;
if isnumeric(choix_mu),
    mu = choix_mu;
else 
    if nargin==6,
        mu = varargin{1};
    else,
        mu = 1;
    end;
end;

%%%OLD%while ((norm(dc_df)/length(dc_df) > epsilon) & (iteration < Nmax)) | (iteration<50),
delta_f = 1;
%while iteration < Nmax,
while ((norm(delta_f)/length(dc_df) > epsilon) & (iteration < Nmax)) | (iteration<50),
   
    iteration = iteration + 1;
    [dc_df,crit_tmp ] = DevKurtR(Y,f);
    
    crit = crit_tmp; %crit(iteration) = crit_tmp;%%%
    
    if choix_mu == 'optim',
        mu = OptStepR(Y,f,dc_df);
        newf = f + mu*dc_df;
        newf = newf/sqrt(newf'*R*newf);
    elseif choix_mu == 'adapt',
        newf = f + mu*dc_df;
        newf = newf/sqrt(newf'*R*newf);
        if crit_tmp>CostKurtR(Y,newf),
            mu = mu/2;
            newf = f+mu*dc_df;
            newf = newf/sqrt(newf'*R*newf);
        end;
    elseif isnumeric(choix_mu),
        newf = f + mu*dc_df;
        newf = newf/sqrt(newf'*R*newf);
    end;
    
    delta_f = f-newf;
    f = newf;
    
end;

%%%crit = mon_cost(Y,f);
f = f'; % f est maintenant un vecteur ligne
W = reshape(f,1,Nobserv,D); %W est sous format standard tableau 1 x Nobserv x D


%plot(crit); disp(choix_mu); disp(mu); pause;