function [G] = MIMOfiltconv(S,H);

% [G] = MIMOfiltconv(S,H);
%
% MIMOfiltconv computes the impulse response of the combined FIR-MIMO filter 
% G=S*H where S and H are FIR-MIMO filters with compatible dimensions
%
% Inputs:
% -------
% S,H: 3D arrays containing the impulse responses of the respective filters
% H_{ij}(k) corresponds to H(i,j,k) and similarly for G and S
%
% Output:
% -------
% G: 3D array containing the impulse response of the combined filter
%
% See also: MIMOfilt
% -------
% Author: Marc Castella
% Created: 12/06/2002
% Modified: 05/07/2006
% Contact: marc.castella@int-evry.fr

% Autres rq:
% G(k) = sum_{l\in Z} S(l)H(k-l) = sum_{p\in Z} S(k-p)H(p)

Ls = size(S,3); Ph = size(H,3);

Lg = Ls+Ph-1;
for k = 0:Lg-1,
    Gtmp = zeros(size(S,1),size(H,2));
    for l = max(0,k-Ph+1):min(Ls-1,k),
        Gtmp = Gtmp + S(:,:,l+1)*H(:,:,k-l+1);
    end;
    G(:,:,k+1) = Gtmp;
end;

%         % G(-1)
%         G(:,:,1) = S(:,:,1)*H(:,:,1);
%         % G(0)
%         G(:,:,2) = S(:,:,2)*H(:,:,1) + S(:,:,1)*H(:,:,2);
%         % G(1)
%         G(:,:,3) = S(:,:,2)*H(:,:,2);

